@@ -611,6 +611,7 @@ class portdbapi(dbapi):
 		# Callers of this method certainly want the same event loop to
 		# be used for all calls.
 		loop = loop or global_event_loop()
+		loop = getattr(loop, '_asyncio_wrapper', loop)
 		future = loop.create_future()
 		cache_me = False
 		if myrepo is not None:
@@ -665,7 +666,7 @@ class portdbapi(dbapi):
 
 		proc = EbuildMetadataPhase(cpv=mycpv,
 			ebuild_hash=ebuild_hash, portdb=self,
-			repo_path=mylocation, scheduler=loop,
+			repo_path=mylocation, scheduler=loop._loop,
 			settings=self.doebuild_settings)
 
 		proc.addExitListener(functools.partial(self._aux_get_return,
@@ -945,7 +946,7 @@ class portdbapi(dbapi):
 						writemsg(_("\nInvalid ebuild version: %s\n") % \
 							os.path.join(oroot, mycp, x), noiselevel=-1)
 						continue
-					d[_pkg_str(mysplit[0]+"/"+pf)] = None
+					d[_pkg_str(mysplit[0]+"/"+pf, db=self)] = None
 		if invalid_category and d:
 			writemsg(_("\n!!! '%s' has a category that is not listed in " \
 				"%setc/portage/categories\n") % \
@@ -1070,7 +1071,7 @@ class portdbapi(dbapi):
 
 					try:
 						pkg_str = _pkg_str(cpv, metadata=metadata,
-							settings=self.settings)
+							settings=self.settings, db=self)
 					except InvalidData:
 						continue
 
