@@ -91,6 +91,30 @@ sandbox_capable = (os.path.isfile(SANDBOX_BINARY) and
 fakeroot_capable = (os.path.isfile(FAKEROOT_BINARY) and
                     os.access(FAKEROOT_BINARY, os.X_OK))
 
+
+def sanitize_fds():
+	"""
+	Set the inheritable flag to False for all open file descriptors,
+	except for those corresponding to stdin, stdout, and stderr. This
+	ensures that any unintentionally inherited file descriptors will
+	not be inherited by child processes.
+	"""
+	if _set_inheritable is not None:
+
+		whitelist = frozenset([
+			sys.__stdin__.fileno(),
+			sys.__stdout__.fileno(),
+			sys.__stderr__.fileno(),
+		])
+
+		for fd in get_open_fds():
+			if fd not in whitelist:
+				try:
+					_set_inheritable(fd, False)
+				except OSError:
+					pass
+
+
 def spawn_bash(mycommand, debug=False, opt_name=None, **keywords):
 	"""
 	Spawns a bash shell running a specific commands
@@ -196,7 +220,8 @@ def cleanup():
 
 def spawn(mycommand, env={}, opt_name=None, fd_pipes=None, returnpid=False,
           uid=None, gid=None, groups=None, umask=None, logfile=None,
-          path_lookup=True, pre_exec=None, close_fds=True, unshare_net=False,
+          path_lookup=True, pre_exec=None,
+          close_fds=(sys.version_info < (3, 4)), unshare_net=False,
           unshare_ipc=False, cgroup=None):
 	"""
 	Spawns a given command.
@@ -228,7 +253,8 @@ def spawn(mycommand, env={}, opt_name=None, fd_pipes=None, returnpid=False,
 	@param pre_exec: A function to be called with no arguments just prior to the exec call.
 	@type pre_exec: callable
 	@param close_fds: If True, then close all file descriptors except those
-		referenced by fd_pipes (default is True).
+		referenced by fd_pipes (default is True for python3.3 and earlier, and False for
+		python3.4 and later due to non-inheritable file descriptor behavior from PEP 446).
 	@type close_fds: Boolean
 	@param unshare_net: If True, networking will be unshared from the spawned process
 	@type unshare_net: Boolean
