@@ -1,4 +1,4 @@
-# Copyright 2010-2015 Gentoo Foundation
+# Copyright 2010-2016 Gentoo Foundation
 # Distributed under the terms of the GNU General Public License v2
 
 import tempfile
@@ -31,7 +31,7 @@ class SleepProcess(ForkProcess):
 
 class IpcDaemonTestCase(TestCase):
 
-	_SCHEDULE_TIMEOUT = 40000 # 40 seconds
+	_SCHEDULE_TIMEOUT = 40 # seconds
 
 	def testIpcDaemon(self):
 		event_loop = global_event_loop()
@@ -103,8 +103,8 @@ class IpcDaemonTestCase(TestCase):
 			# Intentionally short timeout test for EventLoop/AsyncScheduler.
 			# Use a ridiculously long sleep_time_s in case the user's
 			# system is heavily loaded (see bug #436334).
-			sleep_time_s = 600     #600.000 seconds
-			short_timeout_ms = 10  #  0.010 seconds
+			sleep_time_s = 600       # seconds
+			short_timeout_s = 0.010  # seconds
 
 			for i in range(3):
 				exit_command = ExitCommand()
@@ -123,7 +123,7 @@ class IpcDaemonTestCase(TestCase):
 
 				exit_command.reply_hook = exit_command_callback
 				start_time = time.time()
-				self._run(event_loop, task_scheduler, short_timeout_ms)
+				self._run(event_loop, task_scheduler, short_timeout_s)
 
 				hardlock_cleanup(env['PORTAGE_BUILDDIR'],
 					remove_all_locks=True)
@@ -140,19 +140,23 @@ class IpcDaemonTestCase(TestCase):
 				build_dir.unlock()
 			shutil.rmtree(tmpdir)
 
-	def _timeout_callback(self):
-		self._timed_out = True
+	def _timeout_callback(self, task_scheduler):
+		task_scheduler.cancel()
+		self._exit_callback(task_scheduler)
+
+	def _exit_callback(self, task_scheduler):
+		if not self._run_done.done():
+			self._run_done.set_result(True)
 
 	def _run(self, event_loop, task_scheduler, timeout):
-		self._timed_out = False
-		timeout_id = event_loop.timeout_add(timeout, self._timeout_callback)
+		self._run_done = event_loop.create_future()
+		timeout_handle = event_loop.call_later(timeout,
+			self._timeout_callback, task_scheduler)
+		task_scheduler.addExitListener(self._exit_callback)
 
 		try:
 			task_scheduler.start()
-			while not self._timed_out and task_scheduler.poll() is None:
-				event_loop.iteration()
-			if self._timed_out:
-				task_scheduler.cancel()
+			event_loop.run_until_complete(self._run_done)
 			task_scheduler.wait()
 		finally:
-			event_loop.source_remove(timeout_id)
+			timeout_handle.cancel()
