@@ -3,12 +3,15 @@
 
 from __future__ import division
 
+import collections
 import errno
+import functools
 import logging
 import os
 import select
 import signal
 import sys
+import traceback
 
 try:
 	import fcntl
@@ -23,11 +26,11 @@ except ImportError:
 
 import portage
 portage.proxy.lazyimport.lazyimport(globals(),
-	'portage.util.futures.futures:_EventLoopFuture',
+	'portage.util.futures:asyncio',
 	'portage.util.futures.executor.fork:ForkExecutor',
+	'portage.util.futures.unix_events:_PortageEventLoop,_PortageChildWatcher',
 )
 
-from portage import OrderedDict
 from portage.util import writemsg_level
 from portage.util.monotonic import monotonic
 from ..SlotObject import SlotObject
@@ -52,7 +55,7 @@ class EventLoop(object):
 		__slots__ = ("callback", "data", "pid", "source_id")
 
 	class _idle_callback_class(SlotObject):
-		__slots__ = ("args", "callback", "calling", "source_id")
+		__slots__ = ("_args", "_callback", "_cancelled")
 
 	class _io_handler_class(SlotObject):
 		__slots__ = ("args", "callback", "f", "source_id")
@@ -93,6 +96,22 @@ class EventLoop(object):
 			self._callback(*self._args)
 			return False
 
+	class _selector_callback(object):
+		"""
+		Wraps an callback, and always returns True, for callbacks that
+		are supposed to run repeatedly.
+		"""
+		__slots__ = ("_args", "_callbacks")
+
+		def __init__(self, callbacks):
+			self._callbacks = callbacks
+
+		def __call__(self, fd, event):
+			for callback, mask in self._callbacks:
+				if event & mask:
+					callback()
+			return True
+
 	def __init__(self, main=True):
 		"""
 		@param main: If True then this is a singleton instance for use
@@ -102,11 +121,16 @@ class EventLoop(object):
 		@type main: bool
 		"""
 		self._use_signal = main and fcntl is not None
+		self._debug = bool(os.environ.get('PYTHONASYNCIODEBUG'))
 		self._thread_rlock = threading.RLock()
 		self._thread_condition = threading.Condition(self._thread_rlock)
 		self._poll_event_queue = []
 		self._poll_event_handlers = {}
 		self._poll_event_handler_ids = {}
+		# Number of current calls to self.iteration(). A number greater
+		# than 1 indicates recursion, which is not supported by asyncio's
+		# default event loop.
+		self._iteration_depth = 0
 		# Increment id for each new handler.
 		self._event_handler_id = 0
 		# New call_soon callbacks must have an opportunity to
@@ -117,10 +141,11 @@ class EventLoop(object):
 		# If this attribute has changed since the last time that the
 		# call_soon callbacks have been called, then it's not safe to
 		# wait on self._thread_condition without a timeout.
-		self._call_soon_id = 0
-		# Use OrderedDict in order to emulate the FIFO queue behavior
-		# of the AbstractEventLoop.call_soon method.
-		self._idle_callbacks = OrderedDict()
+		self._call_soon_id = None
+		# Use deque, with thread-safe append, in order to emulate the FIFO
+		# queue behavior of the AbstractEventLoop.call_soon method.
+		self._idle_callbacks = collections.deque()
+		self._idle_callbacks_remaining = 0
 		self._timeout_handlers = {}
 		self._timeout_interval = None
 		self._default_executor = None
@@ -167,20 +192,22 @@ class EventLoop(object):
 			self.IO_OUT = PollConstants.POLLOUT
 			self.IO_PRI = PollConstants.POLLPRI
 
+		self._EVENT_READ = self.IO_IN | self.IO_HUP
+		self._EVENT_WRITE = self.IO_OUT
+
 		self._child_handlers = {}
 		self._sigchld_read = None
 		self._sigchld_write = None
 		self._sigchld_src_id = None
 		self._pid = os.getpid()
+		self._asyncio_wrapper = _PortageEventLoop(loop=self)
+		self._asyncio_child_watcher = _PortageChildWatcher(self)
 
 	def create_future(self):
 		"""
-		Create a Future object attached to the loop. This returns
-		an instance of _EventLoopFuture, because EventLoop is currently
-		missing some of the asyncio.AbstractEventLoop methods that
-		asyncio.Future requires.
+		Create a Future object attached to the loop.
 		"""
-		return _EventLoopFuture(loop=self)
+		return asyncio.Future(loop=self._asyncio_wrapper)
 
 	def _new_source_id(self):
 		"""
@@ -245,7 +272,13 @@ class EventLoop(object):
 		@rtype: bool
 		@return: True if events were dispatched.
 		"""
+		self._iteration_depth += 1
+		try:
+			return self._iteration(*args)
+		finally:
+			self._iteration_depth -= 1
 
+	def _iteration(self, *args):
 		may_block = True
 
 		if args:
@@ -266,7 +299,10 @@ class EventLoop(object):
 					events_handled += 1
 				timeouts_checked = True
 
-				call_soon = prev_call_soon_id != self._call_soon_id
+				call_soon = prev_call_soon_id is not self._call_soon_id
+				if self._call_soon_id is not None and self._call_soon_id._cancelled:
+					# Allow garbage collection of cancelled callback.
+					self._call_soon_id = None
 
 				if (not call_soon and not event_handlers
 					and not events_handled and may_block):
@@ -409,8 +445,8 @@ class EventLoop(object):
 					self._sigchld_read, self.IO_IN, self._sigchld_io_cb)
 				signal.signal(signal.SIGCHLD, self._sigchld_sig_cb)
 
-		# poll now, in case the SIGCHLD has already arrived
-		self._poll_child_processes()
+		# poll soon, in case the SIGCHLD has already arrived
+		self.call_soon(self._poll_child_processes)
 		return source_id
 
 	def _sigchld_sig_cb(self, signum, frame):
@@ -469,37 +505,58 @@ class EventLoop(object):
 
 		@type callback: callable
 		@param callback: a function to call
-		@rtype: int
-		@return: an integer ID
+		@return: a handle which can be used to cancel the callback
+			via the source_remove method
+		@rtype: object
 		"""
 		with self._thread_condition:
-			source_id = self._call_soon_id = self._new_source_id()
-			self._idle_callbacks[source_id] = self._idle_callback_class(
-				args=args, callback=callback, source_id=source_id)
+			source_id = self._idle_add(callback, *args)
 			self._thread_condition.notify()
 		return source_id
 
+	def _idle_add(self, callback, *args):
+		"""Like idle_add(), but without thread safety."""
+		# Hold self._thread_condition when assigning self._call_soon_id,
+		# since it might be modified via a thread-safe method.
+		with self._thread_condition:
+			handle = self._call_soon_id = self._idle_callback_class(
+				_args=args, _callback=callback)
+		# This deque append is thread-safe, but it does *not* notify the
+		# loop's thread, so the caller must notify if appropriate.
+		self._idle_callbacks.append(handle)
+		return handle
+
 	def _run_idle_callbacks(self):
 		# assumes caller has acquired self._thread_rlock
 		if not self._idle_callbacks:
 			return False
 		state_change = 0
-		# Iterate of our local list, since self._idle_callbacks can be
-		# modified during the exection of these callbacks.
-		for x in list(self._idle_callbacks.values()):
-			if x.source_id not in self._idle_callbacks:
+		reschedule = []
+		# Use remaining count to avoid calling any newly scheduled callbacks,
+		# since self._idle_callbacks can be modified during the exection of
+		# these callbacks. The remaining count can be reset by recursive
+		# calls to this method. Recursion must remain supported until all
+		# consumers of AsynchronousLock.unlock() have been migrated to the
+		# async_unlock() method, see bug 614108.
+		self._idle_callbacks_remaining = len(self._idle_callbacks)
+
+		while self._idle_callbacks_remaining:
+			self._idle_callbacks_remaining -= 1
+			try:
+				x = self._idle_callbacks.popleft() # thread-safe
+			except IndexError:
+				break
+			if x._cancelled:
 				# it got cancelled while executing another callback
 				continue
-			if x.calling:
-				# don't call it recursively
-				continue
-			x.calling = True
-			try:
-				if not x.callback(*x.args):
-					state_change += 1
-					self.source_remove(x.source_id)
-			finally:
-				x.calling = False
+			if x._callback(*x._args):
+				# Reschedule, but not until after it's called, since
+				# we don't want it to call itself in a recursive call
+				# to this method.
+				self._idle_callbacks.append(x)
+			else:
+				x._cancelled = True
+				state_change += 1
 
 		return bool(state_change)
 
@@ -568,6 +625,102 @@ class EventLoop(object):
 
 		return bool(calls)
 
+	def add_reader(self, fd, callback, *args):
+		"""
+		Start watching the file descriptor for read availability and then
+		call the callback with specified arguments.
+
+		Use functools.partial to pass keywords to the callback.
+		"""
+		handler = self._poll_event_handlers.get(fd)
+		callbacks = [(functools.partial(callback, *args), self._EVENT_READ)]
+		selector_mask = self._EVENT_READ
+		if handler is not None:
+			if not isinstance(handler.callback, self._selector_callback):
+				raise AssertionError("add_reader called with fd "
+					"registered directly via io_add_watch")
+			for item in handler.callback._callbacks:
+				callback, mask = item
+				if mask != self._EVENT_READ:
+					selector_mask |= mask
+					callbacks.append(item)
+			self.source_remove(handler.source_id)
+		self.io_add_watch(fd, selector_mask, self._selector_callback(callbacks))
+
+	def remove_reader(self, fd):
+		"""
+		Stop watching the file descriptor for read availability.
+		"""
+		handler = self._poll_event_handlers.get(fd)
+		if handler is not None:
+			if not isinstance(handler.callback, self._selector_callback):
+				raise AssertionError("remove_reader called with fd "
+					"registered directly via io_add_watch")
+			callbacks = []
+			selector_mask = 0
+			removed = False
+			for item in handler.callback._callbacks:
+				callback, mask = item
+				if mask == self._EVENT_READ:
+					removed = True
+				else:
+					selector_mask |= mask
+					callbacks.append(item)
+			self.source_remove(handler.source_id)
+			if callbacks:
+				self.io_add_watch(fd, selector_mask,
+					self._selector_callback(callbacks))
+			return removed
+		return False
+
+	def add_writer(self, fd, callback, *args):
+		"""
+		Start watching the file descriptor for write availability and then
+		call the callback with specified arguments.
+
+		Use functools.partial to pass keywords to the callback.
+		"""
+		handler = self._poll_event_handlers.get(fd)
+		callbacks = [(functools.partial(callback, *args), self._EVENT_WRITE)]
+		selector_mask = self._EVENT_WRITE
+		if handler is not None:
+			if not isinstance(handler.callback, self._selector_callback):
+				raise AssertionError("add_reader called with fd "
+					"registered directly via io_add_watch")
+			for item in handler.callback._callbacks:
+				callback, mask = item
+				if mask != self._EVENT_WRITE:
+					selector_mask |= mask
+					callbacks.append(item)
+			self.source_remove(handler.source_id)
+		self.io_add_watch(fd, selector_mask, self._selector_callback(callbacks))
+
+	def remove_writer(self, fd):
+		"""
+		Stop watching the file descriptor for write availability.
+		"""
+		handler = self._poll_event_handlers.get(fd)
+		if handler is not None:
+			if not isinstance(handler.callback, self._selector_callback):
+				raise AssertionError("remove_reader called with fd "
+					"registered directly via io_add_watch")
+			callbacks = []
+			selector_mask = 0
+			removed = False
+			for item in handler.callback._callbacks:
+				callback, mask = item
+				if mask == self._EVENT_WRITE:
+					removed = True
+				else:
+					selector_mask |= mask
+					callbacks.append(item)
+			self.source_remove(handler.source_id)
+			if callbacks:
+				self.io_add_watch(fd, selector_mask,
+					self._selector_callback(callbacks))
+			return removed
+		return False
+
 	def io_add_watch(self, f, condition, callback, *args):
 		"""
 		Like glib.io_add_watch(), your function should return False to
@@ -599,6 +752,12 @@ class EventLoop(object):
 		is found and removed, and False if the reg_id is invalid or has
 		already been removed.
 		"""
+		if isinstance(reg_id, self._idle_callback_class):
+			if not reg_id._cancelled:
+				reg_id._cancelled = True
+				return True
+			return False
+
 		x = self._child_handlers.pop(reg_id, None)
 		if x is not None:
 			if not self._child_handlers and self._use_signal:
@@ -608,9 +767,6 @@ class EventLoop(object):
 			return True
 
 		with self._thread_rlock:
-			idle_callback = self._idle_callbacks.pop(reg_id, None)
-			if idle_callback is not None:
-				return True
 			timeout_handler = self._timeout_handlers.pop(reg_id, None)
 			if timeout_handler is not None:
 				if timeout_handler.interval == self._timeout_interval:
@@ -655,6 +811,7 @@ class EventLoop(object):
 		@return: the Future's result
 		@raise: the Future's exception
 		"""
+		future = asyncio.ensure_future(future, loop=self._asyncio_wrapper)
 		while not future.done():
 			self.iteration()
 
@@ -680,11 +837,14 @@ class EventLoop(object):
 		@return: a handle which can be used to cancel the callback
 		@rtype: asyncio.Handle (or compatible)
 		"""
-		return self._handle(self.idle_add(
+		return self._handle(self._idle_add(
 			self._call_soon_callback(callback, args)), self)
 
-	# The call_soon method inherits thread safety from the idle_add method.
-	call_soon_threadsafe = call_soon
+	def call_soon_threadsafe(self, callback, *args):
+		"""Like call_soon(), but thread safe."""
+		# idle_add provides thread safety
+		return self._handle(self.idle_add(
+			self._call_soon_callback(callback, args)), self)
 
 	def time(self):
 		"""Return the time according to the event loop's clock.
@@ -723,6 +883,29 @@ class EventLoop(object):
 		return self._handle(self.timeout_add(
 			delay * 1000, self._call_soon_callback(callback, args)), self)
 
+	def call_at(self, when, callback, *args):
+		"""
+		Arrange for the callback to be called at the given absolute
+		timestamp when (an int or float), using the same time reference as
+		AbstractEventLoop.time().
+
+		This method's behavior is the same as call_later().
+
+		An instance of asyncio.Handle is returned, which can be used to
+		cancel the callback.
+
+		Use functools.partial to pass keywords to the callback.
+
+		@type when: int or float
+		@param when: absolute timestamp when to call callback
+		@type callback: callable
+		@param callback: a function to call
+		@return: a handle which can be used to cancel the callback
+		@rtype: asyncio.Handle (or compatible)
+		"""
+		delta = when - self.time()
+		return self.call_later(delta if delta > 0 else 0, callback, *args)
+
 	def run_in_executor(self, executor, func, *args):
 		"""
 		Arrange for a func to be called in the specified executor.
@@ -746,6 +929,14 @@ class EventLoop(object):
 				self._default_executor = executor
 		return executor.submit(func, *args)
 
+	def is_running(self):
+		"""Return whether the event loop is currently running."""
+		return self._iteration_depth > 0
+
+	def is_closed(self):
+		"""Returns True if the event loop was closed."""
+		return self._poll_obj is None
+
 	def close(self):
 		"""Close the event loop.
 
@@ -763,6 +954,86 @@ class EventLoop(object):
 				close()
 			self._poll_obj = None
 
+	def default_exception_handler(self, context):
+		"""
+		Default exception handler.
+
+		This is called when an exception occurs and no exception
+		handler is set, and can be called by a custom exception
+		handler that wants to defer to the default behavior.
+
+		The context parameter has the same meaning as in
+		`call_exception_handler()`.
+
+		@param context: exception context
+		@type context: dict
+		"""
+		message = context.get('message')
+		if not message:
+			message = 'Unhandled exception in event loop'
+
+		exception = context.get('exception')
+		if exception is not None:
+			exc_info = (type(exception), exception, exception.__traceback__)
+		else:
+			exc_info = False
+
+		log_lines = [message]
+		for key in sorted(context):
+			if key in {'message', 'exception'}:
+				continue
+			value = context[key]
+			if key == 'source_traceback':
+				tb = ''.join(traceback.format_list(value))
+				value = 'Object created at (most recent call last):\n'
+				value += tb.rstrip()
+			elif key == 'handle_traceback':
+				tb = ''.join(traceback.format_list(value))
+				value = 'Handle created at (most recent call last):\n'
+				value += tb.rstrip()
+			else:
+				value = repr(value)
+			log_lines.append('{}: {}'.format(key, value))
+
+		logging.error('\n'.join(log_lines), exc_info=exc_info)
+		os.kill(os.getpid(), signal.SIGTERM)
+
+	def call_exception_handler(self, context):
+		"""
+		Call the current event loop's exception handler.
+
+		The context argument is a dict containing the following keys:
+
+		- 'message': Error message;
+		- 'exception' (optional): Exception object;
+		- 'future' (optional): Future instance;
+		- 'handle' (optional): Handle instance;
+		- 'protocol' (optional): Protocol instance;
+		- 'transport' (optional): Transport instance;
+		- 'socket' (optional): Socket instance;
+		- 'asyncgen' (optional): Asynchronous generator that caused
+								the exception.
+
+		New keys may be introduced in the future.
+
+		@param context: exception context
+		@type context: dict
+		"""
+		self.default_exception_handler(context)
+
+	def get_debug(self):
+		"""
+		Get the debug mode (bool) of the event loop.
+
+		The default value is True if the environment variable
+		PYTHONASYNCIODEBUG is set to a non-empty string, False otherwise.
+		"""
+		return self._debug
+
+	def set_debug(self, enabled):
+		"""Set the debug mode of the event loop."""
+		self._debug = enabled
+
 
 _can_poll_device = None
 
